@@ -3,13 +3,13 @@ package FFI::CheckLib;
 use strict;
 use warnings;
 use File::Spec;
-use Carp qw( croak );
+use Carp qw( croak carp );
 use base qw( Exporter );
 
-our @EXPORT = qw( find_lib assert_lib check_lib check_lib_or_exit );
+our @EXPORT = qw( find_lib assert_lib check_lib check_lib_or_exit find_lib_or_exit );
 
 # ABSTRACT: Check that a library is available for FFI
-our $VERSION = '0.04'; # VERSION
+our $VERSION = '0.05'; # VERSION
 
 
 our $system_path;
@@ -57,10 +57,13 @@ sub _matches
 }
 
 
+my $diagnostic;
+
 sub find_lib
 {
   my(%args) = @_;
   
+  undef $diagnostic;
   croak "find_lib requires lib argument" unless defined $args{lib};
 
   # make arguments be lists.
@@ -133,14 +136,30 @@ sub find_lib
       push @found, $lib->[1];
     }    
   }
+
+  if(%missing)
+  {
+    my @missing = keys %missing;
+    if(@missing > 1)
+    { $diagnostic = "libraries not found: @missing" }
+    else
+    { $diagnostic = "library not found: @missing" }
+  }
+  elsif(%symbols)
+  {
+    my @missing = keys %symbols;
+    $diagnostic = "symbols not found: @missing";
+  }
   
-  %symbols ? () : @found;
+  return if %symbols;
+  return $found[0] unless wantarray;
+  return @found;
 }
 
 
 sub assert_lib
 {
-  die 'library not found' unless check_lib(@_);
+  croak $diagnostic || 'library not found' unless check_lib(@_);
 }
 
 
@@ -148,14 +167,25 @@ sub check_lib_or_exit
 {
   unless(check_lib(@_))
   {
-    # TODO: could probably work on
-    # diagnostics
-    warn "library not found";
+    carp $diagnostic || 'library not found';
     exit;
   }
 }
 
 
+sub find_lib_or_exit
+{
+  my(@libs) = find_lib(@_);
+  unless(@libs)
+  {
+    carp $diagnostic || 'library not found';
+    exit;
+  }
+  return unless @libs;
+  wantarray ? @libs : $libs[0];
+}
+
+
 sub check_lib
 {
   find_lib(@_) ? 1 : 0;
@@ -175,7 +205,7 @@ FFI::CheckLib - Check that a library is available for FFI
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 SYNOPSIS
 
@@ -210,6 +240,10 @@ All of these take the same named parameters and are exported by default.
 
 This will return a list of dynamic libraries, or empty list if none were found.
 
+[version 0.05]
+
+If called in scalar context it will return the first library found.
+
 =head3 lib
 
 Must be either a string with the name of a single library or a reference to an array
@@ -273,11 +307,35 @@ This behaves exactly the same as L<assert_lib|FFI::CheckLib#assert_lib>,
 except that instead of dying, it warns (with exactly the same error message)
 and exists.  This is intended for use in C<Makefile.PL> or C<Build.PL>
 
+=head2 find_lib_or_exit
+
+[version 0.05]
+
+This behaves exactly the same as L<find_lib|FFI::CheckLib#find_lib>,
+except that if the library is not found, it will call exit with an
+appropriate diagnostic.
+
 =head2 check_lib
 
 This behaves exactly the same as L<find_lib|FFI::CheckLib#find_lib>, except that
 it returns true (1) on finding the appropriate libraries or false (0) otherwise.
 
+=head1 SEE ALSO
+
+=over 4
+
+=item L<FFI::Raw>
+
+Call library functions dynamically without a compiler.
+
+=item L<Dist::Zilla::Plugin::FFI::CheckLib>
+
+L<Dist::Zilla> plugin for this module.
+
+=back
+
+=cut
+
 =head1 AUTHOR
 
 Graham Ollis <plicease@cpan.org>
